from subprocess import check_call
import psycopg2
import pandas as pd
from datetime import datetime
import pytz


def query_database():
    query = """
        SELECT 
        scores.commit_sha, scores.tag, competitors."namespace", competitors.project, scores.score, ssi.status_description, c."name", scores.commit_datetime
        FROM public.scores scores 
        left join competitors competitors 
        on competitors.competitor_id = scores.competitor_id 
        left join score_status_ids ssi
        on ssi.status_id = scores.status_id 
        left join competitions c 
        on c.competition_id = scores.competition_id ;
    """
    with psycopg2.connect("") as conn:
        data = pd.read_sql_query(query, conn)
    tz = pytz.timezone("Europe/Zurich")
    data.loc[:, "commit_datetime"] = data.loc[:, "commit_datetime"].dt.tz_convert(tz)
    data = data.rename({
        "commit_sha": "Commit",
        "tag": "Submission tag",
        "namespace": "User",
        "project": "Project",
        "score": "Score",
        "status_description": "Score status",
        "name": "Competition",
        "commit_datetime": "Committed at"
    }, axis=1)
    data = data.loc[:, ["Competition", "Project", "User", "Submission tag", "Score", "Score status", "Committed at", "Commit"]].copy()
    return data

def update_status_only_readme(data):
    tz = pytz.timezone("Europe/Zurich")
    data = query_database()
    updated_at = tz.fromutc(datetime.utcnow()).strftime("%Y-%m-%d %H:%M:%S %Z")
    # remove scores
    data = data.drop(["Score"], axis=1)
    # sort by date and include submissions from the last day only
    now = pd.Timestamp.now(tz=tz)
    ind = data["Committed at"] >= (now - pd.Timedelta("1D"))
    data = data.loc[ind, :].copy()
    # write to readme
    readme_txt = "# S2S AI Competition Submission Status for Last 24 Hours\n"\
        f"{data.to_markdown(index=False)}\n\n"\
        f"Last update: {updated_at}\n\n"\
        "Submissions reported based on date range: "\
        f"{(now - pd.Timedelta('1D')).strftime('%Y-%m-%d %H:%M %Z')} "\
        f"to {now.strftime('%Y-%m-%d %H:%M %Z')}\n"
    check_call(["sh", "-c", "cd s2s-ai-competition-scoring-image && git fetch && git pull"])
    with open("s2s-ai-competition-scoring-image/README.md", "w") as f:
        f.write(readme_txt)
    check_call([
        "sh", 
        "-c", 
        "cd s2s-ai-competition-scoring-image && "\
        f"git add README.md && git commit -m 'Automated status update at: {updated_at}' && "\
        "git push",
    ])

def update_readme():
    tz = pytz.timezone("Europe/Zurich")
    data = query_database()
    updated_at = tz.fromutc(datetime.utcnow()).strftime("%Y-%m-%d %H:%M:%S %Z")
    check_call(["sh", "-c", "git fetch && git pull"])
    data = data.sort_values(["Committed at"], ascending=False)
    data.to_csv("leaderboard_data.csv", index=False)
    data.loc[:, "Committed at"] = data["Committed at"].dt.strftime("%Y-%m-%d %H:%M:%S %Z")
    readme_txt = "# S2S AI Competition Leaderboard\n"\
        f"{data.to_markdown(index=False)}\n\n"\
        f"Last update: {updated_at}"

    with open("README.md", "w") as f:
        f.write(readme_txt)
    
    update_status_only_readme(data)
    
    check_call(["sh", "-c", "git add README.md leaderboard_data.csv s2s-ai-competition-scoring-image"])
    check_call(["sh", "-c", f"git commit -m 'Automated leaderboard update at: {updated_at}'"])
    check_call(["sh", "-c", "git push"])

if __name__ == "__main__":
    update_readme()
